@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2013 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2014 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -18,7 +18,7 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "SDL_config.h"
+#include "../../SDL_internal.h"
 
 #ifdef SDL_JOYSTICK_LINUX
 
@@ -26,7 +26,7 @@
 #error SDL now requires a Linux 2.4+ kernel with /dev/input/event support.
 #endif
 
-/* This is the system specific header for the SDL joystick API */
+/* This is the Linux implementation of the SDL joystick API */
 
 #include <sys/stat.h>
 #include <unistd.h>
@@ -52,124 +52,13 @@
 #define SYN_DROPPED 3
 #endif
 
-/*
- * !!! FIXME: move all the udev stuff to src/core/linux, so I can reuse it
- * !!! FIXME:  for audio hardware disconnects.
- */
-#ifdef HAVE_LIBUDEV_H
-#define SDL_USE_LIBUDEV 1
-#include "SDL_loadso.h"
-#include <libudev.h>
-#include <sys/time.h>
-#include <sys/types.h>
+#include "../../core/linux/SDL_udev.h"
 
-/* we never link directly to libudev. */
-/* !!! FIXME: can we generalize this? ALSA, etc, do the same things. */
-static const char *udev_library = "libudev.so.0";
-static void *udev_handle = NULL;
-
-/* !!! FIXME: this is kinda ugly. */
-static SDL_bool
-load_udev_sym(const char *fn, void **addr)
-{
-    *addr = SDL_LoadFunction(udev_handle, fn);
-    if (*addr == NULL) {
-        /* Don't call SDL_SetError(): SDL_LoadFunction already did. */
-        return SDL_FALSE;
-    }
-
-    return SDL_TRUE;
-}
-
-/* libudev entry points... */
-static const char *(*UDEV_udev_device_get_action)(struct udev_device *) = NULL;
-static const char *(*UDEV_udev_device_get_devnode)(struct udev_device *) = NULL;
-static const char *(*UDEV_udev_device_get_property_value)(struct udev_device *, const char *) = NULL;
-static struct udev_device *(*UDEV_udev_device_new_from_syspath)(struct udev *, const char *) = NULL;
-static void (*UDEV_udev_device_unref)(struct udev_device *) = NULL;
-static int (*UDEV_udev_enumerate_add_match_property)(struct udev_enumerate *, const char *, const char *) = NULL;
-static int (*UDEV_udev_enumerate_add_match_subsystem)(struct udev_enumerate *, const char *) = NULL;
-static struct udev_list_entry *(*UDEV_udev_enumerate_get_list_entry)(struct udev_enumerate *) = NULL;
-static struct udev_enumerate *(*UDEV_udev_enumerate_new)(struct udev *) = NULL;
-static int (*UDEV_udev_enumerate_scan_devices)(struct udev_enumerate *) = NULL;
-static void (*UDEV_udev_enumerate_unref)(struct udev_enumerate *) = NULL;
-static const char *(*UDEV_udev_list_entry_get_name)(struct udev_list_entry *) = NULL;
-static struct udev_list_entry *(*UDEV_udev_list_entry_get_next)(struct udev_list_entry *) = NULL;
-static int (*UDEV_udev_monitor_enable_receiving)(struct udev_monitor *) = NULL;
-static int (*UDEV_udev_monitor_filter_add_match_subsystem_devtype)(struct udev_monitor *, const char *, const char *) = NULL;
-static int (*UDEV_udev_monitor_get_fd)(struct udev_monitor *) = NULL;
-static struct udev_monitor *(*UDEV_udev_monitor_new_from_netlink)(struct udev *, const char *) = NULL;
-static struct udev_device *(*UDEV_udev_monitor_receive_device)(struct udev_monitor *) = NULL;
-static void (*UDEV_udev_monitor_unref)(struct udev_monitor *) = NULL;
-static struct udev *(*UDEV_udev_new)(void) = NULL;
-static void (*UDEV_udev_unref)(struct udev *) = NULL;
-
-static int
-load_udev_syms(void)
-{
-    /* cast funcs to char* first, to please GCC's strict aliasing rules. */
-    #define SDL_UDEV_SYM(x) \
-        if (!load_udev_sym(#x, (void **) (char *) &UDEV_##x)) return -1
-
-    SDL_UDEV_SYM(udev_device_get_action);
-    SDL_UDEV_SYM(udev_device_get_devnode);
-    SDL_UDEV_SYM(udev_device_get_property_value);
-    SDL_UDEV_SYM(udev_device_new_from_syspath);
-    SDL_UDEV_SYM(udev_device_unref);
-    SDL_UDEV_SYM(udev_enumerate_add_match_property);
-    SDL_UDEV_SYM(udev_enumerate_add_match_subsystem);
-    SDL_UDEV_SYM(udev_enumerate_get_list_entry);
-    SDL_UDEV_SYM(udev_enumerate_new);
-    SDL_UDEV_SYM(udev_enumerate_scan_devices);
-    SDL_UDEV_SYM(udev_enumerate_unref);
-    SDL_UDEV_SYM(udev_list_entry_get_name);
-    SDL_UDEV_SYM(udev_list_entry_get_next);
-    SDL_UDEV_SYM(udev_monitor_enable_receiving);
-    SDL_UDEV_SYM(udev_monitor_filter_add_match_subsystem_devtype);
-    SDL_UDEV_SYM(udev_monitor_get_fd);
-    SDL_UDEV_SYM(udev_monitor_new_from_netlink);
-    SDL_UDEV_SYM(udev_monitor_receive_device);
-    SDL_UDEV_SYM(udev_monitor_unref);
-    SDL_UDEV_SYM(udev_new);
-    SDL_UDEV_SYM(udev_unref);
-
-    #undef SDL_UDEV_SYM
-
-    return 0;
-}
-
-static void
-UnloadUDEVLibrary(void)
-{
-    if (udev_handle != NULL) {
-        SDL_UnloadObject(udev_handle);
-        udev_handle = NULL;
-    }
-}
-
-static int
-LoadUDEVLibrary(void)
-{
-    int retval = 0;
-    if (udev_handle == NULL) {
-        udev_handle = SDL_LoadObject(udev_library);
-        if (udev_handle == NULL) {
-            retval = -1;
-            /* Don't call SDL_SetError(): SDL_LoadObject already did. */
-        } else {
-            retval = load_udev_syms();
-            if (retval < 0) {
-                UnloadUDEVLibrary();
-            }
-        }
-    }
-
-    return retval;
-}
-
-static struct udev *udev = NULL;
-static struct udev_monitor *udev_mon = NULL;
-#endif
+static int MaybeAddDevice(const char *path);
+#if SDL_USE_LIBUDEV
+static int MaybeRemoveDevice(const char *path);
+void joystick_udev_callback(SDL_UDEV_deviceevent udev_type, int udev_class, const char *devpath);
+#endif /* SDL_USE_LIBUDEV */
 
 
 /* A linked list of available joysticks */
@@ -196,11 +85,14 @@ static int instance_counter = 0;
 static int
 IsJoystick(int fd, char *namebuf, const size_t namebuflen, SDL_JoystickGUID *guid)
 {
+    struct input_id inpid;
+    Uint16 *guid16 = (Uint16 *) ((char *) &guid->data);
+
+#if !SDL_USE_LIBUDEV
+    /* When udev is enabled we only get joystick devices here, so there's no need to test them */
     unsigned long evbit[NBITS(EV_MAX)] = { 0 };
     unsigned long keybit[NBITS(KEY_MAX)] = { 0 };
     unsigned long absbit[NBITS(ABS_MAX)] = { 0 };
-    struct input_id inpid;
-    Uint16 *guid16 = (Uint16 *) ((char *) &guid->data);
 
     if ((ioctl(fd, EVIOCGBIT(0, sizeof(evbit)), evbit) < 0) ||
         (ioctl(fd, EVIOCGBIT(EV_KEY, sizeof(keybit)), keybit) < 0) ||
@@ -212,6 +104,7 @@ IsJoystick(int fd, char *namebuf, const size_t namebuflen, SDL_JoystickGUID *gui
           test_bit(ABS_X, absbit) && test_bit(ABS_Y, absbit))) {
         return 0;
     }
+#endif
 
     if (ioctl(fd, EVIOCGNAME(namebuflen), namebuf) < 0) {
         return 0;
@@ -246,6 +139,30 @@ IsJoystick(int fd, char *namebuf, const size_t namebuflen, SDL_JoystickGUID *gui
     return 1;
 }
 
+#if SDL_USE_LIBUDEV
+void joystick_udev_callback(SDL_UDEV_deviceevent udev_type, int udev_class, const char *devpath)
+{
+    if (devpath == NULL || !(udev_class & SDL_UDEV_DEVICE_JOYSTICK)) {
+        return;
+    }
+    
+    switch( udev_type )
+    {
+        case SDL_UDEV_DEVICEADDED:
+            MaybeAddDevice(devpath);
+            break;
+            
+        case SDL_UDEV_DEVICEREMOVED:
+            MaybeRemoveDevice(devpath);
+            break;
+            
+        default:
+            break;
+    }
+    
+}
+#endif /* SDL_USE_LIBUDEV */
+
 
 /* !!! FIXME: I would love to dump this code and use libudev instead. */
 static int
@@ -257,6 +174,9 @@ MaybeAddDevice(const char *path)
     char namebuf[128];
     SDL_JoystickGUID guid;
     SDL_joylist_item *item;
+#if !SDL_EVENTS_DISABLED
+    SDL_Event event;
+#endif
 
     if (path == NULL) {
         return -1;
@@ -314,7 +234,23 @@ MaybeAddDevice(const char *path)
         SDL_joylist_tail = item;
     }
 
-    return numjoysticks++;
+    /* Need to increment the joystick count before we post the event */
+    ++numjoysticks;
+
+    /* !!! FIXME: Move this to an SDL_PrivateJoyDeviceAdded() function? */
+#if !SDL_EVENTS_DISABLED
+    event.type = SDL_JOYDEVICEADDED;
+
+    if (SDL_GetEventState(event.type) == SDL_ENABLE) {
+        event.jdevice.which = (numjoysticks - 1);
+        if ( (SDL_EventOK == NULL) ||
+             (*SDL_EventOK) (SDL_EventOKParam, &event) ) {
+            SDL_PushEvent(&event);
+        }
+    }
+#endif /* !SDL_EVENTS_DISABLED */
+
+    return numjoysticks;
 }
 
 #if SDL_USE_LIBUDEV
@@ -324,6 +260,9 @@ MaybeRemoveDevice(const char *path)
 {
     SDL_joylist_item *item;
     SDL_joylist_item *prev = NULL;
+#if !SDL_EVENTS_DISABLED
+    SDL_Event event;
+#endif
 
     if (path == NULL) {
         return -1;
@@ -345,10 +284,26 @@ MaybeRemoveDevice(const char *path)
             if (item == SDL_joylist_tail) {
                 SDL_joylist_tail = prev;
             }
+
+            /* Need to decrement the joystick count before we post the event */
+            --numjoysticks;
+
+            /* !!! FIXME: Move this to an SDL_PrivateJoyDeviceRemoved() function? */
+#if !SDL_EVENTS_DISABLED
+            event.type = SDL_JOYDEVICEREMOVED;
+
+            if (SDL_GetEventState(event.type) == SDL_ENABLE) {
+                event.jdevice.which = item->device_instance;
+                if ( (SDL_EventOK == NULL) ||
+                     (*SDL_EventOK) (SDL_EventOKParam, &event) ) {
+                    SDL_PushEvent(&event);
+                }
+            }
+#endif /* !SDL_EVENTS_DISABLED */
+
             SDL_free(item->path);
             SDL_free(item->name);
             SDL_free(item);
-            numjoysticks--;
             return retval;
         }
         prev = item;
@@ -380,40 +335,19 @@ JoystickInitWithoutUdev(void)
 static int
 JoystickInitWithUdev(void)
 {
-    struct udev_enumerate *enumerate = NULL;
-    struct udev_list_entry *devs = NULL;
-    struct udev_list_entry *item = NULL;
 
-    SDL_assert(udev == NULL);
-    udev = UDEV_udev_new();
-    if (udev == NULL) {
-        return SDL_SetError("udev_new() failed");
+    if (SDL_UDEV_Init() < 0) {
+        return SDL_SetError("Could not initialize UDEV");
     }
 
-    udev_mon = UDEV_udev_monitor_new_from_netlink(udev, "udev");
-    if (udev_mon != NULL) {  /* okay if it's NULL, we just lose hotplugging. */
-        UDEV_udev_monitor_filter_add_match_subsystem_devtype(udev_mon,
-                                                             "input", NULL);
-        UDEV_udev_monitor_enable_receiving(udev_mon);
+    /* Set up the udev callback */
+    if ( SDL_UDEV_AddCallback(joystick_udev_callback) < 0) {
+        SDL_UDEV_Quit();
+        return SDL_SetError("Could not set up joystick <-> udev callback");
     }
-
-    enumerate = UDEV_udev_enumerate_new(udev);
-    if (enumerate == NULL) {
-        return SDL_SetError("udev_enumerate_new() failed");
-    }
-
-    UDEV_udev_enumerate_add_match_subsystem(enumerate, "input");
-    UDEV_udev_enumerate_add_match_property(enumerate, "ID_INPUT_JOYSTICK", "1");
-    UDEV_udev_enumerate_scan_devices(enumerate);
-    devs = UDEV_udev_enumerate_get_list_entry(enumerate);
-    for (item = devs; item; item = UDEV_udev_list_entry_get_next(item)) {
-        const char *path = UDEV_udev_list_entry_get_name(item);
-        struct udev_device *dev = UDEV_udev_device_new_from_syspath(udev, path);
-        MaybeAddDevice(UDEV_udev_device_get_devnode(dev));
-        UDEV_udev_device_unref(dev);
-    }
-
-    UDEV_udev_enumerate_unref(enumerate);
+    
+    /* Force a scan to build the initial device list */
+    SDL_UDEV_Scan();
 
     return numjoysticks;
 }
@@ -439,9 +373,7 @@ SDL_SYS_JoystickInit(void)
     }
 
 #if SDL_USE_LIBUDEV
-    if (LoadUDEVLibrary() == 0) {   /* okay if this fails, FOR NOW. */
-        return JoystickInitWithUdev();
-    }
+    return JoystickInitWithUdev();
 #endif
 
     return JoystickInitWithoutUdev();
@@ -452,99 +384,21 @@ int SDL_SYS_NumJoysticks()
     return numjoysticks;
 }
 
-static SDL_bool
-HotplugUpdateAvailable(void)
-{
-#if SDL_USE_LIBUDEV
-    if (udev_mon != NULL) {
-        const int fd = UDEV_udev_monitor_get_fd(udev_mon);
-        fd_set fds;
-        struct timeval tv;
-
-        FD_ZERO(&fds);
-        FD_SET(fd, &fds);
-        tv.tv_sec = 0;
-        tv.tv_usec = 0;
-        if ((select(fd+1, &fds, NULL, NULL, &tv) > 0) && (FD_ISSET(fd, &fds))) {
-            return SDL_TRUE;
-        }
-    }
-#endif
-
-    return SDL_FALSE;
-}
-
 void SDL_SYS_JoystickDetect()
 {
 #if SDL_USE_LIBUDEV
-    struct udev_device *dev = NULL;
-    const char *devnode = NULL;
-    const char *action = NULL;
-    const char *val = NULL;
-
-    while (HotplugUpdateAvailable()) {
-        dev = UDEV_udev_monitor_receive_device(udev_mon);
-        if (dev == NULL) {
-            break;
-        }
-        val = UDEV_udev_device_get_property_value(dev, "ID_INPUT_JOYSTICK");
-        if ((!val) || (SDL_strcmp(val, "1") != 0)) {
-            continue;
-        }
-
-        action = UDEV_udev_device_get_action(dev);
-        devnode = UDEV_udev_device_get_devnode(dev);
-
-        if (SDL_strcmp(action, "add") == 0) {
-            const int device_index = MaybeAddDevice(devnode);
-            if (device_index != -1) {
-                /* !!! FIXME: Move this to an SDL_PrivateJoyDeviceAdded() function? */
-                #if !SDL_EVENTS_DISABLED
-                SDL_Event event;
-                event.type = SDL_JOYDEVICEADDED;
-
-                if (SDL_GetEventState(event.type) == SDL_ENABLE) {
-                    event.jdevice.which = device_index;
-                    if ( (SDL_EventOK == NULL) ||
-                         (*SDL_EventOK) (SDL_EventOKParam, &event) ) {
-                        SDL_PushEvent(&event);
-                    }
-                }
-                #endif /* !SDL_EVENTS_DISABLED */
-            }
-        } else if (SDL_strcmp(action, "remove") == 0) {
-            const int inst = MaybeRemoveDevice(devnode);
-            if (inst != -1) {
-                /* !!! FIXME: Move this to an SDL_PrivateJoyDeviceRemoved() function? */
-                #if !SDL_EVENTS_DISABLED
-                SDL_Event event;
-                event.type = SDL_JOYDEVICEREMOVED;
-
-                if (SDL_GetEventState(event.type) == SDL_ENABLE) {
-                    event.jdevice.which = inst;
-                    if ( (SDL_EventOK == NULL) ||
-                         (*SDL_EventOK) (SDL_EventOKParam, &event) ) {
-                        SDL_PushEvent(&event);
-                    }
-                }
-                #endif /* !SDL_EVENTS_DISABLED */
-            }
-        }
-        UDEV_udev_device_unref(dev);
-    }
+    SDL_UDEV_Poll();
 #endif
+    
 }
 
 SDL_bool SDL_SYS_JoystickNeedsPolling()
 {
-    /*
-     * This results in a select() call, so technically we're polling to
-     *  decide if we should poll, but I think this function is here because
-     *  Windows has to do an enormous amount of work to detect new sticks,
-     *  whereas libudev just needs to see if there's more data available on
-     *  a socket...so this should be acceptable, I hope.
-     */
-    return HotplugUpdateAvailable();
+#if SDL_USE_LIBUDEV
+    return SDL_TRUE;
+#endif
+    
+    return SDL_FALSE;
 }
 
 static SDL_joylist_item *
@@ -655,10 +509,11 @@ ConfigJoystick(SDL_Joystick * joystick, int fd)
             if (test_bit(i, absbit)) {
                 struct input_absinfo absinfo;
 
-                if (ioctl(fd, EVIOCGABS(i), &absinfo) < 0)
+                if (ioctl(fd, EVIOCGABS(i), &absinfo) < 0) {
                     continue;
+                }
 #ifdef DEBUG_INPUT_EVENTS
-                printf("Joystick has absolute axis: %x\n", i);
+                printf("Joystick has absolute axis: 0x%.2x\n", i);
                 printf("Values = { %d, %d, %d, %d, %d }\n",
                        absinfo.value, absinfo.minimum, absinfo.maximum,
                        absinfo.fuzz, absinfo.flat);
@@ -685,9 +540,17 @@ ConfigJoystick(SDL_Joystick * joystick, int fd)
         }
         for (i = ABS_HAT0X; i <= ABS_HAT3Y; i += 2) {
             if (test_bit(i, absbit) || test_bit(i + 1, absbit)) {
+                struct input_absinfo absinfo;
+
+                if (ioctl(fd, EVIOCGABS(i), &absinfo) < 0) {
+                    continue;
+                }
 #ifdef DEBUG_INPUT_EVENTS
                 printf("Joystick has hat %d\n", (i - ABS_HAT0X) / 2);
-#endif
+                printf("Values = { %d, %d, %d, %d, %d }\n",
+                       absinfo.value, absinfo.minimum, absinfo.maximum,
+                       absinfo.fuzz, absinfo.flat);
+#endif /* DEBUG_INPUT_EVENTS */
                 ++joystick->nhats;
             }
         }
@@ -772,7 +635,7 @@ SDL_bool SDL_SYS_JoystickAttached(SDL_Joystick *joystick)
     return !joystick->closed && (joystick->hwdata->item != NULL);
 }
 
-static __inline__ void
+static SDL_INLINE void
 HandleHat(SDL_Joystick * stick, Uint8 hat, int axis, int value)
 {
     struct hwdata_hat *the_hat;
@@ -798,14 +661,14 @@ HandleHat(SDL_Joystick * stick, Uint8 hat, int axis, int value)
     }
 }
 
-static __inline__ void
+static SDL_INLINE void
 HandleBall(SDL_Joystick * stick, Uint8 ball, int axis, int value)
 {
     stick->hwdata->balls[ball].axis[axis] += value;
 }
 
 
-static __inline__ int
+static SDL_INLINE int
 AxisCorrect(SDL_Joystick * joystick, int which, int value)
 {
     struct axis_correct *correct;
@@ -834,7 +697,7 @@ AxisCorrect(SDL_Joystick * joystick, int which, int value)
     return value;
 }
 
-static __inline__ void
+static SDL_INLINE void
 PollAllValues(SDL_Joystick * joystick)
 {
     struct input_absinfo absinfo;
@@ -872,7 +735,7 @@ PollAllValues(SDL_Joystick * joystick)
     }
 }
 
-static __inline__ void
+static SDL_INLINE void
 HandleInputEvents(SDL_Joystick * joystick)
 {
     struct input_event events[32];
@@ -1011,15 +874,8 @@ SDL_SYS_JoystickQuit(void)
     instance_counter = 0;
 
 #if SDL_USE_LIBUDEV
-    if (udev_mon != NULL) {
-        UDEV_udev_monitor_unref(udev_mon);
-        udev_mon = NULL;
-    }
-    if (udev != NULL) {
-        UDEV_udev_unref(udev);
-        udev = NULL;
-    }
-    UnloadUDEVLibrary();
+    SDL_UDEV_DelCallback(joystick_udev_callback);
+    SDL_UDEV_Quit();
 #endif
 }
 
