@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2013 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2014 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -18,7 +18,7 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "SDL_config.h"
+#include "../../SDL_internal.h"
 
 #if SDL_VIDEO_DRIVER_DIRECTFB
 
@@ -132,7 +132,7 @@ MotionAllMice(_THIS, int x, int y)
         SDL_Mouse *mouse = SDL_GetMouse(index);
         mouse->x = mouse->last_x = x;
         mouse->y = mouse->last_y = y;
-        /*SDL_SendMouseMotion(devdata->mouse_id[index], 0, x, y, 0);*/
+        /* SDL_SendMouseMotion(devdata->mouse_id[index], 0, x, y, 0); */
     }
 #endif
 }
@@ -233,7 +233,7 @@ ProcessWindowEvent(_THIS, SDL_Window *sdlwin, DFBWindowEvent * evt)
         case DWET_KEYDOWN:
             if (!devdata->use_linux_input) {
                 DirectFB_TranslateKey(_this, evt, &keysym, &unicode);
-                /*printf("Scancode %d  %d %d\n", keysym.scancode, evt->key_code, evt->key_id);*/
+                /* printf("Scancode %d  %d %d\n", keysym.scancode, evt->key_code, evt->key_id); */
                 SDL_SendKeyboardKey_ex(0, SDL_PRESSED, keysym.scancode);
                 if (SDL_EventState(SDL_TEXTINPUT, SDL_QUERY)) {
                     SDL_zero(text);
@@ -369,7 +369,7 @@ ProcessInputEvent(_THIS, DFBInputEvent * ievt)
         case DIET_KEYPRESS:
             kbd_idx = KbdIndex(_this, ievt->device_id);
             DirectFB_TranslateKeyInputEvent(_this, ievt, &keysym, &unicode);
-            /*printf("Scancode %d  %d %d\n", keysym.scancode, evt->key_code, evt->key_id); */
+            /* printf("Scancode %d  %d %d\n", keysym.scancode, evt->key_code, evt->key_id); */
             SDL_SendKeyboardKey_ex(kbd_idx, SDL_PRESSED, keysym.scancode);
             if (SDL_EventState(SDL_TEXTINPUT, SDL_QUERY)) {
                 SDL_zero(text);
@@ -742,7 +742,7 @@ DirectFB_InitKeyboard(_THIS)
 void
 DirectFB_QuitKeyboard(_THIS)
 {
-    /*SDL_DFB_DEVICEDATA(_this); */
+    /* SDL_DFB_DEVICEDATA(_this); */
 
     SDL_KeyboardQuit();
 
