@@ -190,10 +190,8 @@ int render_testPrimitives (void *arg)
    _compare(referenceSurface, ALLOWABLE_ERROR_OPAQUE );
 
    /* Clean up. */
-   if (referenceSurface != NULL) {
-      SDL_FreeSurface(referenceSurface);
-      referenceSurface = NULL;
-   }
+   SDL_FreeSurface(referenceSurface);
+   referenceSurface = NULL;
 
    return TEST_COMPLETED;
 }
@@ -333,10 +331,8 @@ int render_testPrimitivesBlend (void *arg)
    _compare(referenceSurface, ALLOWABLE_ERROR_BLENDED );
 
    /* Clean up. */
-   if (referenceSurface != NULL) {
-      SDL_FreeSurface(referenceSurface);
-      referenceSurface = NULL;
-   }
+   SDL_FreeSurface(referenceSurface);
+   referenceSurface = NULL;
 
    return TEST_COMPLETED;
 }
@@ -400,10 +396,8 @@ render_testBlit(void *arg)
 
    /* Clean up. */
    SDL_DestroyTexture( tface );
-   if (referenceSurface != NULL) {
-      SDL_FreeSurface(referenceSurface);
-      referenceSurface = NULL;
-   }
+   SDL_FreeSurface(referenceSurface);
+   referenceSurface = NULL;
 
    return TEST_COMPLETED;
 }
@@ -470,10 +464,8 @@ render_testBlitColor (void *arg)
 
    /* Clean up. */
    SDL_DestroyTexture( tface );
-   if (referenceSurface != NULL) {
-      SDL_FreeSurface(referenceSurface);
-      referenceSurface = NULL;
-   }
+   SDL_FreeSurface(referenceSurface);
+   referenceSurface = NULL;
 
    return TEST_COMPLETED;
 }
@@ -543,10 +535,8 @@ render_testBlitAlpha (void *arg)
 
    /* Clean up. */
    SDL_DestroyTexture( tface );
-   if (referenceSurface != NULL) {
-      SDL_FreeSurface(referenceSurface);
-      referenceSurface = NULL;
-   }
+   SDL_FreeSurface(referenceSurface);
+   referenceSurface = NULL;
 
    return TEST_COMPLETED;
 }
@@ -655,37 +645,29 @@ render_testBlitBlend (void *arg)
    _testBlitBlendMode( tface, SDL_BLENDMODE_NONE );
    referenceSurface = SDLTest_ImageBlitBlendNone();
    _compare(referenceSurface, ALLOWABLE_ERROR_OPAQUE );
-   if (referenceSurface != NULL) {
-      SDL_FreeSurface(referenceSurface);
-      referenceSurface = NULL;
-   }
+   SDL_FreeSurface(referenceSurface);
+   referenceSurface = NULL;
 
    /* Test Blend. */
    _testBlitBlendMode( tface, SDL_BLENDMODE_BLEND );
    referenceSurface = SDLTest_ImageBlitBlend();
    _compare(referenceSurface, ALLOWABLE_ERROR_BLENDED );
-   if (referenceSurface != NULL) {
-      SDL_FreeSurface(referenceSurface);
-      referenceSurface = NULL;
-   }
+   SDL_FreeSurface(referenceSurface);
+   referenceSurface = NULL;
 
    /* Test Add. */
    _testBlitBlendMode( tface, SDL_BLENDMODE_ADD );
    referenceSurface = SDLTest_ImageBlitBlendAdd();
    _compare(referenceSurface, ALLOWABLE_ERROR_BLENDED );
-   if (referenceSurface != NULL) {
-      SDL_FreeSurface(referenceSurface);
-      referenceSurface = NULL;
-   }
+   SDL_FreeSurface(referenceSurface);
+   referenceSurface = NULL;
 
    /* Test Mod. */
    _testBlitBlendMode( tface, SDL_BLENDMODE_MOD);
    referenceSurface = SDLTest_ImageBlitBlendMod();
    _compare(referenceSurface, ALLOWABLE_ERROR_BLENDED );
-   if (referenceSurface != NULL) {
-      SDL_FreeSurface(referenceSurface);
-      referenceSurface = NULL;
-   }
+   SDL_FreeSurface(referenceSurface);
+   referenceSurface = NULL;
 
    /* Clear surface. */
    _clearScreen();
@@ -733,10 +715,8 @@ render_testBlitBlend (void *arg)
    /* Check to see if final image matches. */
    referenceSurface = SDLTest_ImageBlitBlendAll();
    _compare(referenceSurface, ALLOWABLE_ERROR_BLENDED);
-   if (referenceSurface != NULL) {
-      SDL_FreeSurface(referenceSurface);
-      referenceSurface = NULL;
-   }
+   SDL_FreeSurface(referenceSurface);
+   referenceSurface = NULL;
 
    return TEST_COMPLETED;
 }
@@ -995,12 +975,8 @@ _compare(SDL_Surface *referenceSurface, int allowable_error)
    SDLTest_AssertCheck(result == 0, "Validate result from SDLTest_CompareSurfaces, expected: 0, got: %i", result);
 
    /* Clean up. */
-   if (pixels != NULL) {
-       SDL_free(pixels);
-   }
-   if (testSurface != NULL) {
-       SDL_FreeSurface(testSurface);
-   }
+   SDL_free(pixels);
+   SDL_FreeSurface(testSurface);
 }
 
 /**
